window.onload = function() {
  let dragElements = document.querySelectorAll('.drag');

  for (let el of dragElements) {
    focusElement(el);
    collapseElement(el);
    dragElement(el);
    resizeElement(el);
  }

  // Resize adapted from Lazaro@SO.
  // https://stackoverflow.com/questions/8960193/how-to-make-html-element-resizable-using-pure-javascript
  function resizeElement(elem) {
    let resizer = document.querySelector('#' + elem.id + 'resize');
    if (resizer) {
      resizer.addEventListener('pointerdown', initResize);

      function initResize(e) {
        resizer.setPointerCapture(event.pointerId);
        resizer.addEventListener('pointermove', resize);
        resizer.addEventListener('pointerup', stopResize);
      }

      function resize(e) {
        let shiftX = e.clientX - resizer.getBoundingClientRect().left;
        let shiftY = e.clientY - resizer.getBoundingClientRect().top;
        elem.style.width = e.clientX - elem.offsetLeft + shiftX/2 + 'px';
        elem.style.height = e.clientY - elem.offsetTop + shiftY/2 + 'px';
      }

      function stopResize(e) {
        resizer.removeEventListener('pointermove', resize);
        resizer.removeEventListener('pointerup', stopResize);
      }
      resizer.ondragstart = () => false;
    }
  }

  function focusElement(elem) {
    elem.onpointerdown = () => {
      for (let el of dragElements) el.style.zIndex = '9';
      elem.style.zIndex = '100';
    };
  }

  function collapseElement(elem){
    let clBut = document.querySelector('#' + elem.id + 'button');
    let resizer = document.querySelector('#' + elem.id + 'resize');
    if (clBut) {
      clBut.onclick = () => {
        elem.classList.toggle('collapsed');
        clBut.firstChild.classList.toggle('toggled');
        if (resizer) resizer.classList.toggle('removed');
      };
    }
  }

  // Drag adapted from The Modern JavaScript Tutorial.
  // https://javascript.info/
  function dragElement(elem) {
    let head = document.querySelector('#' + elem.id + 'head');
    if (head) {
      head.onpointerdown = function (e) {
        let shiftX = e.clientX - head.getBoundingClientRect().left;
        let shiftY = e.clientY - head.getBoundingClientRect().top;

        moveAt(e.pageX, e.pageY);

        function moveAt(pageX, pageY) {
          elem.style.left = pageX - shiftX + 'px';
          elem.style.top = pageY - shiftY + 'px';
        }
        function onPointerMove(e) {
          moveAt(e.pageX, e.pageY);
        }

        document.addEventListener('pointermove', onPointerMove);
        document.onpointerup = function() {
          document.removeEventListener('pointermove', onPointerMove);
          document.onpointerup = null;
        };
      };
      head.ondragstart = () => false;
    }
  }
}
